/*
 *
 * $Id: k3bdevicemanager.h,v 1.26 2004/01/21 10:20:11 trueg Exp $
 * Copyright (C) 2003 Sebastian Trueg <trueg@k3b.org>
 *
 * This file is part of the K3b project.
 * Copyright (C) 1998-2004 Sebastian Trueg <trueg@k3b.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * See the file "COPYING" for the exact licensing terms.
 */


#ifndef K3BDEVICEMANAGER_H
#define K3BDEVICEMANAGER_H

#include <qobject.h>
#include <qstring.h>
#include <qstringlist.h>
#include <qmemarray.h>
#include <qptrlist.h>

#include <kdebug.h>
#ifdef __FreeBSD__
#include <qdatetime.h>
#include <qvaluelist.h>
#endif

class KProcess;
class KConfig;
class K3bExternalBin;


namespace K3bCdDevice {

  class CdDevice;

  class DeviceManager : public QObject
    {
      Q_OBJECT

    public:
      DeviceManager( QObject* parent = 0, const char* name = 0 );
      ~DeviceManager();

      CdDevice* deviceByName( const QString& );

      CdDevice* findDevice( int bus, int id, int lun );
      CdDevice* findDevice( const QString& devicename );

      /**
       * Before getting the devices do a @ref scanbus().
       * @return List of all cd writer devices.
       * @deprecated use cdWriter
       */
      QPtrList<CdDevice>& burningDevices();

      /**
       * Note that all burning devices can also be used as
       * reading device and are not present in this list.
       * Before getting the devices do a @ref scanbus().
       * @return List of all reader devices without writer devices.
       * @deprecated use cdReader
       **/
      QPtrList<CdDevice>& readingDevices();

      QPtrList<CdDevice>& allDevices();

      QPtrList<CdDevice>& cdWriter();
      QPtrList<CdDevice>& cdReader();
      QPtrList<CdDevice>& dvdWriter();
      QPtrList<CdDevice>& dvdReader();

      /** writes to stderr **/
      void printDevices();

      /**
       * Returns number of found devices
       **/
      int scanbus();

      void scanFstab();

      /**
       * Reads the device information from the config file.
       */
      bool readConfig( KConfig* );

      bool saveConfig( KConfig* );

      /**
       * Clears the writers and readers list of devices.
       */
      void clear();

      /**
       * add a new device like "/dev/mebecdrom" to be sensed
       * by the deviceManager.
       */
      CdDevice* addDevice( const QString& );

    private slots:
      void slotCollectStdout( KProcess*, char* data, int len );

    private:
      bool testForCdrom( const QString& );
      bool determineBusIdLun( const QString &dev, int& bus, int& id, int& lun );
      QString resolveSymLink( const QString& path );

      int m_foundDevices;

      QString m_processOutput;

      class Private;
      Private* d;
#ifdef __FreeBSD__
      void bsd_scan_devices(bool sudo);
      QDateTime m_lastRefresh;
      class BSDDevice
      {
      public:
          BSDDevice() : m_bus(-1), m_target(-1), m_lun(-1) {}
          BSDDevice(const QString & name, const QString & passDevice, const QString & vendor,
                    const QString & product, const QString & revision, int bus, int target, int lun) :
              m_name(name), m_passDevice(passDevice), m_vendor(vendor), m_product(product),
              m_revision(revision), m_bus(bus), m_target(target), m_lun(lun) {}
          BSDDevice(const BSDDevice & src) :
              m_name(src.m_name), m_passDevice(src.m_passDevice), m_vendor(src.m_vendor), m_product(src.m_product),
              m_revision(src.m_revision), m_bus(src.m_bus), m_target(src.m_target), m_lun(src.m_lun) {}
          const BSDDevice & operator= (const BSDDevice & src)
          { m_name = src.m_name; m_passDevice = src.m_passDevice; m_vendor = src.m_vendor;
            m_product = src.m_product; m_revision = src.m_revision; m_bus = src.m_bus; m_target = src.m_target;
            m_lun = src.m_lun; return *this; }
          QString m_name, m_passDevice, m_vendor, m_product, m_revision;
          int m_bus, m_target, m_lun;
      };
      QValueList<BSDDevice> m_devices;
#endif
    };
}

typedef K3bCdDevice::DeviceManager K3bDeviceManager;
#endif
